﻿//================================================================
// Anisotropy image generator script for Photoshop CC.
// Copyright (c) 2018 Y.Kurozumi.
// http://techip.info/
// ----------------------------------------------------------------
// Ver1.00  2018/07/24  First release.
//================================================================

// jshint esversion: 6
// jshint evil: true
// #target photoshop

// Constants
const RESOLUTION = 72.000000;
const CANVAS_SIZE = 4096;
const BORDER_WIDTH = 16;
const SPIN_BLUR_AMOUNT = 360;
const POST_BLUR_AMOUNT = 100;

// Values.
var _dlg;
var _docName;
var _escRulerUnits;
var _escTypeUnits;
var _escDisplayDialogs;

try {
    // Save settings.
    _escRulerUnits = app.preferences.rulerUnits;
    _escTypeUnits = app.preferences.typeUnits;
    _escDisplayDialogs = app.displayDialogs;

    // Change settings.
    app.preferences.rulerUnits = Units.PIXELS;
    app.preferences.typeUnits = TypeUnits.PIXELS;
    app.displayDialogs = DialogModes.NO;

    // Main process.
    buildUI();
    _dlg.show();

} catch (e) {
    alert(e);

} finally {
    // Restore settings.
    if (undefined != _escRulerUnits) app.displayDialogs = _escDisplayDialogs;
    if (undefined != _escTypeUnits) app.preferences.typeUnits = _escTypeUnits;
    if (undefined != _escDisplayDialogs) app.displayDialogs = _escDisplayDialogs;
}


//================================================================
// Functions.
//================================================================

//----------------------------------------------------------------
// UI Setting.
//----------------------------------------------------------------
function buildUI() {
    // Dialog settings.
    const MARGIN = 8;
    const LBL_WIDTH = 100;
    const BTN_WIDTH = 155;

    _dlg = new Window("dialog", "Anisotropy image generator", { x: 0, y: 0, width: 330, height: 200 });
    _dlg.center();

    var posY = MARGIN;
    _dlg.add("statictext", { x: MARGIN, y: posY, width: LBL_WIDTH, height: 20 }, "Image type");
    _dlg.drpType = _dlg.add("dropdownlist", { x: MARGIN * 2 + LBL_WIDTH, y: posY, width: 200, height: 20 }, "DropDownList");
    _dlg.drpType.itms = [];
    _dlg.drpType.itms.push(_dlg.drpType.add("item", "Anisotropy direction map"));
    _dlg.drpType.itms.push(_dlg.drpType.add("item", "Anisotropy bump map (Noise)"));
    _dlg.drpType.itms.push(_dlg.drpType.add("item", "Anisotropy bump map (Film Grain)"));
    _dlg.drpType.itms.push(_dlg.drpType.add("item", "Anisotropy bump map (Circle)"));
    posY += 20;

    _dlg.add("statictext", { x: MARGIN, y: posY, width: LBL_WIDTH, height: 20 }, "Canvas size");
    _dlg.txtSize = _dlg.add("edittext", { x: MARGIN * 2 + LBL_WIDTH, y: posY, width: 60, height: 20 }, CANVAS_SIZE);
    posY += 20;

    _dlg.add("statictext", { x: MARGIN, y: posY, width: LBL_WIDTH, height: 20 }, "Spin blur amount");
    _dlg.txtSpinBlurAmount = _dlg.add("edittext", { x: MARGIN * 2 + LBL_WIDTH, y: posY, width: 60, height: 20 }, SPIN_BLUR_AMOUNT);
    _dlg.add("statictext", { x: MARGIN * 3 + LBL_WIDTH + 60, y: posY, width: LBL_WIDTH, height: 20 }, "(Range: 0..360)");
    posY += 20;

    _dlg.add("statictext", { x: MARGIN, y: posY, width: LBL_WIDTH, height: 20 }, "Border width");
    _dlg.txtWidth = _dlg.add("edittext", { x: MARGIN * 2 + LBL_WIDTH, y: posY, width: 60, height: 20 }, BORDER_WIDTH);
    posY += 30;

    _dlg.chkRange = _dlg.add("checkbox", { x: MARGIN, y: posY, width: 100, height: 20 }, "Draw circle to image corner (not recommended)");
    posY += 20;

    _dlg.chkRBlur = _dlg.add("checkbox", { x: MARGIN, y: posY, width: 100, height: 20 }, "Post spin blur");
    _dlg.txtPostBlurAmount = _dlg.add("edittext", { x: MARGIN * 2 + 100, y: posY, width: 60, height: 20 }, POST_BLUR_AMOUNT);
    _dlg.add("statictext", { x: MARGIN * 3 + LBL_WIDTH + 60, y: posY, width: LBL_WIDTH, height: 20 }, "(Range: 0..100)");
    posY += 40;

    _dlg.btnOk = _dlg.add("button", { x: MARGIN + 0, y: posY, width: BTN_WIDTH, height: 20 }, "OK");
    _dlg.btnCancel = _dlg.add("button", { x: MARGIN + BTN_WIDTH, y: posY, width: BTN_WIDTH, height: 20 }, "Close");


    //----------------------------------------------------------------
    // TextBox changed.
    //----------------------------------------------------------------
    _dlg.txtSize.onChange = function () {
        _dlg.txtSize.text = eval(_dlg.txtSize.text);
    };
    _dlg.txtWidth.onChange = function () {
        _dlg.txtWidth.text = eval(_dlg.txtWidth.text);
    };
    _dlg.txtSpinBlurAmount.onChange = function () {
        _dlg.txtSpinBlurAmount.text = eval(_dlg.txtSpinBlurAmount.text);
    };
    _dlg.txtPostBlurAmount.onChange = function () {
        _dlg.txtPostBlurAmount.text = eval(_dlg.txtPostBlurAmount.text);
    };

    //----------------------------------------------------------------
    // Image type changed.
    //----------------------------------------------------------------
    _dlg.drpType.onChange = function () {
        // alert(_dlg.drpType.selection.index);
        switch (_dlg.drpType.selection.index) {
            case 0: // Anisotropy direction map
                _docName = "AnisotropyColorMap";
                _dlg.txtWidth.enabled = false;
                _dlg.txtSpinBlurAmount.enabled = false;
                _dlg.chkRange.enabled = false;
                _dlg.chkRBlur.value = false;
                _dlg.txtPostBlurAmount.enabled = false;
                break;

            case 1: // Anisotropy bump map (Noise)
                _docName = "AnisotropyNoiseBump";
                _dlg.txtWidth.enabled = false;
                _dlg.txtSpinBlurAmount.enabled = true;
                _dlg.chkRange.enabled = false;
                _dlg.chkRBlur.value = true;
                _dlg.txtPostBlurAmount.enabled = true;
                break;

            case 2: // Anisotropy bump map (Film Grain)
                _docName = "AnisotropyGrainBump";
                _dlg.txtWidth.enabled = false;
                _dlg.txtSpinBlurAmount.enabled = true;
                _dlg.chkRange.enabled = false;
                _dlg.chkRBlur.value = true;
                _dlg.txtPostBlurAmount.enabled = true;
                break;

            case 3: // Anisotropy bump map (Circle)
                _docName = "AnisotropyCircleBump";
                _dlg.txtWidth.enabled = true;
                _dlg.txtSpinBlurAmount.enabled = false;
                _dlg.chkRange.enabled = true;
                _dlg.chkRBlur.value = true;
                _dlg.txtPostBlurAmount.enabled = true;
                break;
        }
    };

    //----------------------------------------------------------------
    // Post spin blur check box.
    //----------------------------------------------------------------
    _dlg.chkRBlur.onClick = function () {
        _dlg.txtPostBlurAmount.enabled = _dlg.chkRBlur.value;
    };

    //----------------------------------------------------------------
    // OK Button.
    //----------------------------------------------------------------
    _dlg.btnOk.onClick = function () {
        // _dlg.close();

        // Canvas settings.
        var canvasSize = eval(_dlg.txtSize.text);
        var canvasR = canvasSize / 2.0;

        // Circle parameter settings.
        var width = eval(_dlg.txtWidth.text);
        var step = width * 2.0;
        var counts = Math.ceil(canvasR / step);
        if (_dlg.chkRange.value) {
            counts = Math.ceil(canvasR * Math.SQRT2 / step);
        }

        // Create new document.
        var doc = app.documents.add(canvasSize, canvasSize, RESOLUTION, _docName, NewDocumentMode.RGB, DocumentFill.WHITE, 1.0, BitsPerChannelType.SIXTEEN);
        var bgLayer = doc.backgroundLayer;

        // Draw image.
        var imageType = _dlg.drpType.selection.index;
        switch (imageType) {
            case 0: // Anisotropy direction map
                drawAnisotropyGradation(canvasR, canvasR, canvasR);
                break;

            case 1: // Anisotropy bump map (Noise)
                var amount1 = _dlg.txtSpinBlurAmount.text;
                addNoiseFilter(400);
                spinBlurFilter(canvasR, canvasR, canvasSize, canvasSize, canvasR * Math.SQRT2, amount1);
                cameraRawAdjustContrast();
                autoLevels();
                break;

            case 2: // Anisotropy bump map (Film Grain)
                var amount2 = _dlg.txtSpinBlurAmount.text;
                fill(127.0, 127.0, 127.0); // Gray
                cameraRawAddGrain();
                //addNoiseFilter(400);
                spinBlurFilter(canvasR, canvasR, canvasSize, canvasSize, canvasR * Math.SQRT2, amount2);
                cameraRawAdjustContrast();
                autoLevels();
                break;

            case 3: // Anisotropy bump map (Circle)
                bgLayer.invert(); // White -> Black
                for (var i = 0; i < counts; i++) {
                    drawCircle(canvasR, canvasR, step * i - width, width);
                }
                break;
        }

        // Flattens all layers.
        doc.flatten();

        // Radial spin blur filter.
        if (_dlg.chkRBlur.value) {
            var postBlurAmount = _dlg.txtPostBlurAmount.text;
            radialBlurFilter(postBlurAmount);
        }

        // Refresh.
        app.refresh();
    };

    //----------------------------------------------------------------
    // Cancel button.
    //----------------------------------------------------------------
    _dlg.btnCancel.onClick = function () {
        _dlg.close();
    };

    // Initial UI settings.
    _dlg.drpType.selection = 0;
}

//----------------------------------------------------------------
// Draw anisotropy gradation.
//----------------------------------------------------------------
function drawAnisotropyGradation(x, y, r) {
    var list1 = new ActionList();
    var list2 = new ActionList();
    var dsc01 = new ActionDescriptor();
    var dsc02 = new ActionDescriptor();
    var dsc03 = new ActionDescriptor();
    var dsc04 = new ActionDescriptor();
    var dsc05 = new ActionDescriptor();
    var dsc06 = new ActionDescriptor();
    var dsc07 = new ActionDescriptor();
    var dsc08 = new ActionDescriptor();
    var dsc09 = new ActionDescriptor();
    var dsc10 = new ActionDescriptor();
    var dsc11 = new ActionDescriptor();
    var dsc12 = new ActionDescriptor();
    var dsc13 = new ActionDescriptor();
    var dsc14 = new ActionDescriptor();
    var dsc15 = new ActionDescriptor();
    var dsc16 = new ActionDescriptor();
    var dsc17 = new ActionDescriptor();
    var dsc18 = new ActionDescriptor();
    var dsc19 = new ActionDescriptor();
    var dsc20 = new ActionDescriptor();
    var dsc21 = new ActionDescriptor();
    var dsc22 = new ActionDescriptor();
    var dsc23 = new ActionDescriptor();
    var dsc24 = new ActionDescriptor();

    var idAng = charIDToTypeID("#Ang");
    var idAngl = charIDToTypeID("Angl");
    var idBl = charIDToTypeID("Bl  ");
    var idBrgh = charIDToTypeID("Brgh");
    var idClr = charIDToTypeID("Clr ");
    var idClrs = charIDToTypeID("Clrs");
    var idClrt = charIDToTypeID("Clrt");
    var idClry = charIDToTypeID("Clry");
    var idCstS = charIDToTypeID("CstS");
    var idDthr = charIDToTypeID("Dthr");
    var idFrom = charIDToTypeID("From");
    var idGrad = charIDToTypeID("Grad");
    var idGrdF = charIDToTypeID("GrdF");
    var idGrdn = charIDToTypeID("Grdn");
    var idGrdT = charIDToTypeID("GrdT");
    var idGrn = charIDToTypeID("Grn ");
    var idH = charIDToTypeID("H   ");
    var idHrzn = charIDToTypeID("Hrzn");
    var idHSBC = charIDToTypeID("HSBC");
    var idIntr = charIDToTypeID("Intr");
    var idLctn = charIDToTypeID("Lctn");
    var idMdpn = charIDToTypeID("Mdpn");
    var idNm = charIDToTypeID("Nm  ");
    var idOpct = charIDToTypeID("Opct");
    var idPnt = charIDToTypeID("Pnt ");
    var idPrc = charIDToTypeID("#Prc");
    var idPxl = charIDToTypeID("#Pxl");
    var idRd = charIDToTypeID("Rd  ");
    var idRGBC = charIDToTypeID("RGBC");
    var idStrt = charIDToTypeID("Strt");
    var idT = charIDToTypeID("T   ");
    var idTrnS = charIDToTypeID("TrnS");
    var idType = charIDToTypeID("Type");
    var idUsMs = charIDToTypeID("UsMs");
    var idUsrS = charIDToTypeID("UsrS");
    var idVrtc = charIDToTypeID("Vrtc");

    dsc02.putUnitDouble(idHrzn, idPxl, x);
    dsc02.putUnitDouble(idVrtc, idPxl, y);
    dsc01.putObject(idFrom, idPnt, dsc02);
    dsc03.putUnitDouble(idHrzn, idPxl, x);
    dsc03.putUnitDouble(idVrtc, idPxl, y - r);
    dsc01.putObject(idT, idPnt, dsc03);
    dsc01.putEnumerated(idType, idGrdT, idAngl);
    dsc01.putBoolean(idDthr, true);
    dsc01.putBoolean(idUsMs, true);
    dsc04.putEnumerated(idGrdF, idGrdF, idCstS);
    dsc04.putDouble(idIntr, 4096.000000);
    dsc06.putDouble(idRd, 128.000000);
    dsc06.putDouble(idGrn, 0.000000);
    dsc06.putDouble(idBl, 0.000000);
    dsc05.putObject(idClr, idRGBC, dsc06);
    dsc05.putEnumerated(idType, idClry, idUsrS);
    dsc05.putInteger(idLctn, 0);
    dsc05.putInteger(idMdpn, 50);
    list1.putObject(idClrt, dsc05);
    dsc08.putUnitDouble(idH, idAng, 0.000000);
    dsc08.putDouble(idStrt, 100.000000);
    dsc08.putDouble(idBrgh, 100.000000);
    dsc07.putObject(idClr, idHSBC, dsc08);
    dsc07.putEnumerated(idType, idClry, idUsrS);
    dsc07.putInteger(idLctn, 512);
    dsc07.putInteger(idMdpn, 50);
    list1.putObject(idClrt, dsc07);
    dsc10.putDouble(idRd, 255.000000);
    dsc10.putDouble(idGrn, 0.000000);
    dsc10.putDouble(idBl, 128.000000);
    dsc09.putObject(idClr, idRGBC, dsc10);
    dsc09.putEnumerated(idType, idClry, idUsrS);
    dsc09.putInteger(idLctn, 1013);
    dsc09.putInteger(idMdpn, 50);
    list1.putObject(idClrt, dsc09);
    dsc12.putDouble(idRd, 255.000000);
    dsc12.putDouble(idGrn, 0.000000);
    dsc12.putDouble(idBl, 255.000000);
    dsc11.putObject(idClr, idRGBC, dsc12);
    dsc11.putEnumerated(idType, idClry, idUsrS);
    dsc11.putInteger(idLctn, 1536);
    dsc11.putInteger(idMdpn, 50);
    list1.putObject(idClrt, dsc11);
    dsc14.putDouble(idRd, 128.000000);
    dsc14.putDouble(idGrn, 0.000000);
    dsc14.putDouble(idBl, 255.000000);
    dsc13.putObject(idClr, idRGBC, dsc14);
    dsc13.putEnumerated(idType, idClry, idUsrS);
    dsc13.putInteger(idLctn, 2048);
    dsc13.putInteger(idMdpn, 50);
    list1.putObject(idClrt, dsc13);
    dsc16.putDouble(idRd, 0.000000);
    dsc16.putDouble(idGrn, 0.000000);
    dsc16.putDouble(idBl, 255.000000);
    dsc15.putObject(idClr, idRGBC, dsc16);
    dsc15.putEnumerated(idType, idClry, idUsrS);
    dsc15.putInteger(idLctn, 2560);
    dsc15.putInteger(idMdpn, 50);
    list1.putObject(idClrt, dsc15);
    dsc18.putDouble(idRd, 0.000000);
    dsc18.putDouble(idGrn, 0.000000);
    dsc18.putDouble(idBl, 128.000000);
    dsc17.putObject(idClr, idRGBC, dsc18);
    dsc17.putEnumerated(idType, idClry, idUsrS);
    dsc17.putInteger(idLctn, 3061);
    dsc17.putInteger(idMdpn, 50);
    list1.putObject(idClrt, dsc17);
    dsc20.putDouble(idRd, 0.000000);
    dsc20.putDouble(idGrn, 0.000000);
    dsc20.putDouble(idBl, 0.000000);
    dsc19.putObject(idClr, idRGBC, dsc20);
    dsc19.putEnumerated(idType, idClry, idUsrS);
    dsc19.putInteger(idLctn, 3584);
    dsc19.putInteger(idMdpn, 50);
    list1.putObject(idClrt, dsc19);
    dsc22.putDouble(idRd, 128.000000);
    dsc22.putDouble(idGrn, 0.000000);
    dsc22.putDouble(idBl, 0.000000);
    dsc21.putObject(idClr, idRGBC, dsc22);
    dsc21.putEnumerated(idType, idClry, idUsrS);
    dsc21.putInteger(idLctn, 4096);
    dsc21.putInteger(idMdpn, 50);
    list1.putObject(idClrt, dsc21);
    dsc04.putList(idClrs, list1);
    dsc23.putUnitDouble(idOpct, idPrc, 100.000000);
    dsc23.putInteger(idLctn, 0);
    dsc23.putInteger(idMdpn, 50);
    list2.putObject(idTrnS, dsc23);
    dsc24.putUnitDouble(idOpct, idPrc, 100.000000);
    dsc24.putInteger(idLctn, 4096);
    dsc24.putInteger(idMdpn, 50);
    list2.putObject(idTrnS, dsc24);
    dsc04.putList(charIDToTypeID("Trns"), list2);
    dsc01.putObject(idGrad, idGrdn, dsc04);
    executeAction(idGrdn, dsc01, DialogModes.NO);
}

// ----------------------------------------------------------------
// Draw circle shape.
//----------------------------------------------------------------
function drawCircle(x, y, r, width) {
    var ref1 = new ActionReference();
    var dsc1 = new ActionDescriptor();
    var dsc2 = new ActionDescriptor();
    var dsc3 = new ActionDescriptor();
    var dsc4 = new ActionDescriptor();
    var dsc5 = new ActionDescriptor();
    var dsc6 = new ActionDescriptor();

    var idPxl = charIDToTypeID("#Pxl");

    var descWhite = new ActionDescriptor();
    descWhite.putDouble(charIDToTypeID("Rd  "), 255.000000);
    descWhite.putDouble(charIDToTypeID("Grn "), 255.000000);
    descWhite.putDouble(charIDToTypeID("Bl  "), 255.000000);

    ref1.putClass(stringIDToTypeID("contentLayer"));
    dsc1.putReference(charIDToTypeID("null"), ref1);
    dsc3.putObject(charIDToTypeID("Clr "), charIDToTypeID("RGBC"), descWhite);
    dsc2.putObject(charIDToTypeID("Type"), stringIDToTypeID("solidColorLayer"), dsc3);
    dsc4.putInteger(stringIDToTypeID("unitValueQuadVersion"), 1);
    dsc4.putUnitDouble(charIDToTypeID("Top "), idPxl, y - r);
    dsc4.putUnitDouble(charIDToTypeID("Left"), idPxl, x - r);
    dsc4.putUnitDouble(charIDToTypeID("Btom"), idPxl, y + r);
    dsc4.putUnitDouble(charIDToTypeID("Rght"), idPxl, x + r);
    dsc2.putObject(charIDToTypeID("Shp "), charIDToTypeID("Elps"), dsc4);
    dsc5.putInteger(stringIDToTypeID("strokeStyleVersion"), 2);
    dsc5.putBoolean(stringIDToTypeID("strokeEnabled"), true);
    dsc5.putBoolean(stringIDToTypeID("fillEnabled"), false);
    dsc5.putUnitDouble(stringIDToTypeID("strokeStyleLineWidth"), idPxl, width);
    dsc5.putUnitDouble(stringIDToTypeID("strokeStyleLineDashOffset"), charIDToTypeID("#Pnt"), 0.000000);
    dsc5.putDouble(stringIDToTypeID("strokeStyleMiterLimit"), 100.000000);
    dsc5.putEnumerated(stringIDToTypeID("strokeStyleLineCapType"), stringIDToTypeID("strokeStyleLineCapType"), stringIDToTypeID("strokeStyleButtCap"));
    dsc5.putEnumerated(stringIDToTypeID("strokeStyleLineJoinType"), stringIDToTypeID("strokeStyleLineJoinType"), stringIDToTypeID("strokeStyleMiterJoin"));
    dsc5.putEnumerated(stringIDToTypeID("strokeStyleLineAlignment"), stringIDToTypeID("strokeStyleLineAlignment"), stringIDToTypeID("strokeStyleAlignCenter"));
    dsc5.putBoolean(stringIDToTypeID("strokeStyleScaleLock"), false);
    dsc5.putBoolean(stringIDToTypeID("strokeStyleStrokeAdjust"), false);
    dsc5.putList(stringIDToTypeID("strokeStyleLineDashSet"), new ActionList());
    dsc5.putEnumerated(stringIDToTypeID("strokeStyleBlendMode"), charIDToTypeID("BlnM"), charIDToTypeID("Nrml"));
    dsc5.putUnitDouble(stringIDToTypeID("strokeStyleOpacity"), charIDToTypeID("#Prc"), 100.000000);
    dsc6.putObject(charIDToTypeID("Clr "), charIDToTypeID("RGBC"), descWhite);
    dsc5.putObject(stringIDToTypeID("strokeStyleContent"), stringIDToTypeID("solidColorLayer"), dsc6);
    dsc5.putDouble(stringIDToTypeID("strokeStyleResolution"), RESOLUTION);
    dsc2.putObject(stringIDToTypeID("strokeStyle"), stringIDToTypeID("strokeStyle"), dsc5);
    dsc1.putObject(charIDToTypeID("Usng"), stringIDToTypeID("contentLayer"), dsc2);
    dsc1.putInteger(charIDToTypeID("LyrI"), 2);
    executeAction(charIDToTypeID("Mk  "), dsc1, DialogModes.NO);
}

//----------------------------------------------------------------
// Radial blur filter (Spin).
// amount: 0..100
//----------------------------------------------------------------
function radialBlurFilter(amount) {
    var dsc1 = new ActionDescriptor();
    var dsc2 = new ActionDescriptor();

    dsc1.putInteger(charIDToTypeID("Amnt"), amount);
    dsc1.putEnumerated(charIDToTypeID("BlrM"), charIDToTypeID("BlrM"), charIDToTypeID("Spn "));
    dsc1.putEnumerated(charIDToTypeID("BlrQ"), charIDToTypeID("BlrQ"), charIDToTypeID("Bst "));
    dsc2.putDouble(charIDToTypeID("Hrzn"), 0.500000);
    dsc2.putDouble(charIDToTypeID("Vrtc"), 0.500000);
    dsc1.putObject(charIDToTypeID("Cntr"), charIDToTypeID("Pnt "), dsc2);
    executeAction(charIDToTypeID("RdlB"), dsc1, DialogModes.NO);
}

//----------------------------------------------------------------
// Adjust Levels (Auto)
//----------------------------------------------------------------
function autoLevels() {
    var lst1 = new ActionList();
    var ref1 = new ActionReference();
    var dsc1 = new ActionDescriptor();
    var dsc2 = new ActionDescriptor();

    var idLvls = charIDToTypeID("Lvls");
    var idpresetKind = stringIDToTypeID("presetKind");
    var idpresetKindType = stringIDToTypeID("presetKindType");
    var idpresetKindDefault = stringIDToTypeID("presetKindDefault");
    var idAdjs = charIDToTypeID("Adjs");
    var idChnl = charIDToTypeID("Chnl");
    var idCmps = charIDToTypeID("Cmps");
    var idautoMachineLearning = stringIDToTypeID("autoMachineLearning");
    var idautoFaces = stringIDToTypeID("autoFaces");
    var idLvlA = charIDToTypeID("LvlA");

    dsc1.putEnumerated(idpresetKind, idpresetKindType, idpresetKindDefault);
    ref1.putEnumerated(idChnl, idChnl, idCmps);
    dsc2.putReference(idChnl, ref1);
    dsc2.putBoolean(idautoMachineLearning, true);
    dsc2.putBoolean(idautoFaces, true);
    lst1.putObject(idLvlA, dsc2);
    dsc1.putList(idAdjs, lst1);
    executeAction(idLvls, dsc1, DialogModes.NO);
}

//----------------------------------------------------------------
// Fill (Fill point = [0,0])
//----------------------------------------------------------------
function fill(r, g, b) {
    var ref1 = new ActionReference();
    var dsc1 = new ActionDescriptor();
    var dsc2 = new ActionDescriptor();
    var dsc3 = new ActionDescriptor();
    var dsc4 = new ActionDescriptor();

    var idsetd = charIDToTypeID("setd");
    var idnull = charIDToTypeID("null");
    var idClr = charIDToTypeID("Clr ");
    var idFrgC = charIDToTypeID("FrgC");
    var idT = charIDToTypeID("T   ");
    var idRd = charIDToTypeID("Rd  ");
    var idGrn = charIDToTypeID("Grn ");
    var idBl = charIDToTypeID("Bl  ");
    var idRGBC = charIDToTypeID("RGBC");
    var idSrce = charIDToTypeID("Srce");
    var idFl = charIDToTypeID("Fl  ");
    var idFrom = charIDToTypeID("From");
    var idHrzn = charIDToTypeID("Hrzn");
    var idPxl = charIDToTypeID("#Pxl");
    var idVrtc = charIDToTypeID("Vrtc");
    var idPnt = charIDToTypeID("Pnt ");
    var idTlrn = charIDToTypeID("Tlrn");
    var idAntA = charIDToTypeID("AntA");
    var idUsng = charIDToTypeID("Usng");
    var idFlCn = charIDToTypeID("FlCn");

    ref1.putProperty(idClr, idFrgC);
    dsc1.putReference(idnull, ref1);
    dsc2.putDouble(idRd, r);
    dsc2.putDouble(idGrn, g);
    dsc2.putDouble(idBl, b);
    dsc1.putObject(idT, idRGBC, dsc2);
    executeAction(idsetd, dsc1, DialogModes.NO);

    dsc4.putUnitDouble(idHrzn, idPxl, 0.000000);
    dsc4.putUnitDouble(idVrtc, idPxl, 0.000000);
    dsc3.putObject(idFrom, idPnt, dsc4);
    dsc3.putInteger(idTlrn, 32);
    dsc3.putBoolean(idAntA, true);
    dsc3.putEnumerated(idUsng, idFlCn, idFrgC);
    executeAction(idFl, dsc3, DialogModes.NO);
}

//----------------------------------------------------------------
// Camera Raw: Noise filter
//----------------------------------------------------------------
function cameraRawAddGrain() {
    var lst1 = new ActionList();
    var lst2 = new ActionList();
    var lst3 = new ActionList();
    var lst4 = new ActionList();
    var dsc1 = new ActionDescriptor();

    var idAdobeCameraRawFilter = stringIDToTypeID("Adobe Camera Raw Filter");
    var idBkonetwo = charIDToTypeID("Bk12");
    var idClonetwo = charIDToTypeID("Cl12");
    var idCronetwo = charIDToTypeID("Cr12");
    var idCrv = charIDToTypeID("Crv ");
    var idCrvB = charIDToTypeID("CrvB");
    var idCrvG = charIDToTypeID("CrvG");
    var idCrvR = charIDToTypeID("CrvR");
    var idCst = charIDToTypeID("Cst ");
    var idCtoG = charIDToTypeID("CtoG");
    var idDhze = charIDToTypeID("Dhze");
    var idExonetwo = charIDToTypeID("Ex12");
    var idGRNA = charIDToTypeID("GRNA");
    var idGRNF = charIDToTypeID("GRNF");
    var idGRNS = charIDToTypeID("GRNS");
    var idHionetwo = charIDToTypeID("Hi12");
    var idLNR = charIDToTypeID("LNR ");
    var idLPEn = charIDToTypeID("LPEn");
    var idMDis = charIDToTypeID("MDis");
    var idPC_D = charIDToTypeID("PC_D");
    var idPC_H = charIDToTypeID("PC_H");
    var idPC_L = charIDToTypeID("PC_L");
    var idPC_one = charIDToTypeID("PC_1");
    var idPC_S = charIDToTypeID("PC_S");
    var idPC_three = charIDToTypeID("PC_3");
    var idPC_two = charIDToTypeID("PC_2");
    var idPCVA = charIDToTypeID("PCVA");
    var idPerA = charIDToTypeID("PerA");
    var idPerH = charIDToTypeID("PerH");
    var idPerR = charIDToTypeID("PerR");
    var idPerS = charIDToTypeID("PerS");
    var idPerU = charIDToTypeID("PerU");
    var idPerV = charIDToTypeID("PerV");
    var idPerX = charIDToTypeID("PerX");
    var idPerY = charIDToTypeID("PerY");
    var idSett = charIDToTypeID("Sett");
    var idShonetwo = charIDToTypeID("Sh12");
    var idShpD = charIDToTypeID("ShpD");
    var idShpM = charIDToTypeID("ShpM");
    var idShpR = charIDToTypeID("ShpR");
    var idShrp = charIDToTypeID("Shrp");
    var idSTB = charIDToTypeID("STB ");
    var idSTHH = charIDToTypeID("STHH");
    var idSTHS = charIDToTypeID("STHS");
    var idSTSH = charIDToTypeID("STSH");
    var idSTSS = charIDToTypeID("STSS");
    var idTMMs = charIDToTypeID("TMMs");
    var idVigA = charIDToTypeID("VigA");
    var idWhonetwo = charIDToTypeID("Wh12");

    dsc1.putEnumerated(idSett, idSett, idCst);
    dsc1.putBoolean(idCtoG, false);
    dsc1.putInteger(idShrp, 0);
    dsc1.putInteger(idLNR, 0);
    dsc1.putInteger(idVigA, 0);
    dsc1.putInteger(idSTSH, 0);
    dsc1.putInteger(idSTSS, 0);
    dsc1.putInteger(idSTHH, 0);
    dsc1.putInteger(idSTHS, 0);
    dsc1.putInteger(idSTB, 0);
    dsc1.putInteger(idPC_S, 0);
    dsc1.putInteger(idPC_D, 0);
    dsc1.putInteger(idPC_L, 0);
    dsc1.putInteger(idPC_H, 0);
    dsc1.putInteger(idPC_one, 25);
    dsc1.putInteger(idPC_two, 50);
    dsc1.putInteger(idPC_three, 75);
    dsc1.putDouble(idShpR, 1.000000);
    dsc1.putInteger(idShpD, 25);
    dsc1.putInteger(idShpM, 0);
    dsc1.putInteger(idPCVA, 0);
    dsc1.putInteger(idGRNA, 100);
    dsc1.putInteger(idGRNS, 100);
    dsc1.putInteger(idGRNF, 100);
    dsc1.putInteger(idLPEn, 0);
    dsc1.putInteger(idMDis, 0);
    dsc1.putInteger(idPerV, 0);
    dsc1.putInteger(idPerH, 0);
    dsc1.putDouble(idPerR, 0.000000);
    dsc1.putInteger(idPerS, 100);
    dsc1.putInteger(idPerA, 0);
    dsc1.putInteger(idPerU, 0);
    dsc1.putDouble(idPerX, 0.000000);
    dsc1.putDouble(idPerY, 0.000000);
    dsc1.putDouble(idExonetwo, 0.000000);
    dsc1.putInteger(idCronetwo, 0);
    dsc1.putInteger(idHionetwo, 0);
    dsc1.putInteger(idShonetwo, 0);
    dsc1.putInteger(idWhonetwo, 0);
    dsc1.putInteger(idBkonetwo, 0);
    dsc1.putInteger(idClonetwo, 0);
    dsc1.putInteger(idDhze, 0);
    dsc1.putInteger(idTMMs, 0);
    lst1.putInteger(0);
    lst1.putInteger(255);
    dsc1.putList(idCrv, lst1);
    lst2.putInteger(0);
    lst2.putInteger(255);
    dsc1.putList(idCrvR, lst2);
    lst3.putInteger(0);
    lst3.putInteger(255);
    dsc1.putList(idCrvG, lst3);
    lst4.putInteger(0);
    lst4.putInteger(255);
    dsc1.putList(idCrvB, lst4);
    executeAction(idAdobeCameraRawFilter, dsc1, DialogModes.NO);
}

//----------------------------------------------------------------
// Blur Gallery: Spin blur.
// amount: 0..360
//----------------------------------------------------------------
function spinBlurFilter(x, y, w, h, r, amount) {
    var lst1 = new ActionList();
    var lst2 = new ActionList();
    var dsc1 = new ActionDescriptor();
    var dsc2 = new ActionDescriptor();
    var dsc3 = new ActionDescriptor();

    var idblurbHighQuality = stringIDToTypeID("blurbHighQuality");
    var idblurbSaveMaskChannel = stringIDToTypeID("blurbSaveMaskChannel");
    var idblurbTransform = stringIDToTypeID("blurbTransform");
    var idblurbType = stringIDToTypeID("blurbType");
    var idblurbWheelSpinBlurAmount = stringIDToTypeID("blurbWheelSpinBlurAmount");
    var idblurbWheelSpinBlurEffectApplied = stringIDToTypeID("blurbWheelSpinBlurEffectApplied");
    var idblurbWheelSpinCorneronex = stringIDToTypeID("blurbWheelSpinCorner1x");
    var idblurbWheelSpinCorneroney = stringIDToTypeID("blurbWheelSpinCorner1y");
    var idblurbWheelSpinCornerthreex = stringIDToTypeID("blurbWheelSpinCorner3x");
    var idblurbWheelSpinCornerthreey = stringIDToTypeID("blurbWheelSpinCorner3y");
    var idblurbWheelSpinCornertwox = stringIDToTypeID("blurbWheelSpinCorner2x");
    var idblurbWheelSpinCornertwoy = stringIDToTypeID("blurbWheelSpinCorner2y");
    var idblurbWheelSpinCornerzerox = stringIDToTypeID("blurbWheelSpinCorner0x");
    var idblurbWheelSpinCornerzeroy = stringIDToTypeID("blurbWheelSpinCorner0y");
    var idblurbWheelSpinFeatherone = stringIDToTypeID("blurbWheelSpinFeather1");
    var idblurbWheelSpinFeatherthree = stringIDToTypeID("blurbWheelSpinFeather3");
    var idblurbWheelSpinFeathertwo = stringIDToTypeID("blurbWheelSpinFeather2");
    var idblurbWheelSpinFeatherzero = stringIDToTypeID("blurbWheelSpinFeather0");
    var idblurbWheelSpinPhysicalCenterX = stringIDToTypeID("blurbWheelSpinPhysicalCenterX");
    var idblurbWheelSpinPhysicalCenterY = stringIDToTypeID("blurbWheelSpinPhysicalCenterY");
    var idblurbWidget = stringIDToTypeID("blurbWidget");
    var idblurbWidgetEffectEnabled = stringIDToTypeID("blurbWidgetEffectEnabled");
    var idblurbWidgetList = stringIDToTypeID("blurbWidgetList");
    var idblurbWidgetListtwo = stringIDToTypeID("blurbWidgetList2");
    var idblurbWidgetLocationX = stringIDToTypeID("blurbWidgetLocationX");
    var idblurbWidgetLocationY = stringIDToTypeID("blurbWidgetLocationY");
    var idblurbWidgetSelected = stringIDToTypeID("blurbWidgetSelected");
    var idblurbWidgetType = stringIDToTypeID("blurbWidgetType");
    var idBtom = charIDToTypeID("Btom");
    var idLeft = charIDToTypeID("Left");
    var idRctn = charIDToTypeID("Rctn");
    var idreferenceRect = stringIDToTypeID("referenceRect");
    var idRght = charIDToTypeID("Rght");
    var idTop = charIDToTypeID("Top ");
    var idVrsM = charIDToTypeID("VrsM");
    var idVrsN = charIDToTypeID("VrsN");

    dsc1.putBoolean(idblurbType, true);
    dsc1.putInteger(idVrsM, 1);
    dsc1.putInteger(idVrsN, 0);
    dsc2.putInteger(idTop, 0);
    dsc2.putInteger(idLeft, 0);
    dsc2.putInteger(idBtom, h);
    dsc2.putInteger(idRght, w);
    dsc1.putObject(idreferenceRect, idRctn, dsc2);
    dsc1.putBoolean(idblurbWheelSpinBlurEffectApplied, true);
    dsc1.putBoolean(idblurbSaveMaskChannel, false);
    dsc1.putBoolean(idblurbHighQuality, true);
    dsc3.putInteger(idblurbWidgetType, 5);
    dsc3.putDouble(idblurbWidgetLocationX, x);
    dsc3.putDouble(idblurbWidgetLocationY, y);
    dsc3.putBoolean(idblurbWidgetSelected, true);
    dsc3.putBoolean(idblurbWidgetEffectEnabled, true);
    dsc3.putDouble(idblurbWheelSpinBlurAmount, amount);
    dsc3.putDouble(idblurbWheelSpinPhysicalCenterX, x);
    dsc3.putDouble(idblurbWheelSpinPhysicalCenterY, y);
    dsc3.putDouble(idblurbWheelSpinFeatherzero, 0.000000);
    dsc3.putDouble(idblurbWheelSpinFeatherone, 0.000000);
    dsc3.putDouble(idblurbWheelSpinFeathertwo, 0.000000);
    dsc3.putDouble(idblurbWheelSpinFeatherthree, 0.000000);
    r += 64; // Add margin.
    dsc3.putDouble(idblurbWheelSpinCornerzerox, x - r);
    dsc3.putDouble(idblurbWheelSpinCornerzeroy, y - r);
    dsc3.putDouble(idblurbWheelSpinCorneronex, x + r);
    dsc3.putDouble(idblurbWheelSpinCorneroney, y - r);
    dsc3.putDouble(idblurbWheelSpinCornertwox, x + r);
    dsc3.putDouble(idblurbWheelSpinCornertwoy, y + r);
    dsc3.putDouble(idblurbWheelSpinCornerthreex, x - r);
    dsc3.putDouble(idblurbWheelSpinCornerthreey, y + r);
    lst1.putObject(idblurbWidget, dsc3);
    dsc1.putList(idblurbWidgetListtwo, lst1);
    dsc1.putList(idblurbWidgetList, lst2);
    executeAction(idblurbTransform, dsc1, DialogModes.NO);
}

//----------------------------------------------------------------
// Camera raw: Adjust contrast and more...
//----------------------------------------------------------------
function cameraRawAdjustContrast() {
    var lst1 = new ActionList();
    var lst2 = new ActionList();
    var lst3 = new ActionList();
    var lst4 = new ActionList();
    var dsc1 = new ActionDescriptor();

    var idAdobeCameraRawFilter = stringIDToTypeID("Adobe Camera Raw Filter");
    var idBkonetwo = charIDToTypeID("Bk12");
    var idClonetwo = charIDToTypeID("Cl12");
    var idCronetwo = charIDToTypeID("Cr12");
    var idCrv = charIDToTypeID("Crv ");
    var idCrvB = charIDToTypeID("CrvB");
    var idCrvG = charIDToTypeID("CrvG");
    var idCrvR = charIDToTypeID("CrvR");
    var idCst = charIDToTypeID("Cst ");
    var idCtoG = charIDToTypeID("CtoG");
    var idDhze = charIDToTypeID("Dhze");
    var idExonetwo = charIDToTypeID("Ex12");
    var idGRNA = charIDToTypeID("GRNA");
    var idHionetwo = charIDToTypeID("Hi12");
    var idLNR = charIDToTypeID("LNR ");
    var idLPEn = charIDToTypeID("LPEn");
    var idMDis = charIDToTypeID("MDis");
    var idPC_D = charIDToTypeID("PC_D");
    var idPC_H = charIDToTypeID("PC_H");
    var idPC_L = charIDToTypeID("PC_L");
    var idPC_one = charIDToTypeID("PC_1");
    var idPC_S = charIDToTypeID("PC_S");
    var idPC_three = charIDToTypeID("PC_3");
    var idPC_two = charIDToTypeID("PC_2");
    var idPCVA = charIDToTypeID("PCVA");
    var idPerA = charIDToTypeID("PerA");
    var idPerH = charIDToTypeID("PerH");
    var idPerR = charIDToTypeID("PerR");
    var idPerS = charIDToTypeID("PerS");
    var idPerU = charIDToTypeID("PerU");
    var idPerV = charIDToTypeID("PerV");
    var idPerX = charIDToTypeID("PerX");
    var idPerY = charIDToTypeID("PerY");
    var idPrVe = charIDToTypeID("PrVe");
    var idSett = charIDToTypeID("Sett");
    var idShonetwo = charIDToTypeID("Sh12");
    var idShpD = charIDToTypeID("ShpD");
    var idShpM = charIDToTypeID("ShpM");
    var idShpR = charIDToTypeID("ShpR");
    var idShrp = charIDToTypeID("Shrp");
    var idSTB = charIDToTypeID("STB ");
    var idSTHH = charIDToTypeID("STHH");
    var idSTHS = charIDToTypeID("STHS");
    var idSTSH = charIDToTypeID("STSH");
    var idSTSS = charIDToTypeID("STSS");
    var idTMMs = charIDToTypeID("TMMs");
    var idVigA = charIDToTypeID("VigA");
    var idWhonetwo = charIDToTypeID("Wh12");

    dsc1.putEnumerated(idSett, idSett, idCst);
    dsc1.putBoolean(idCtoG, false);
    dsc1.putInteger(idShrp, 0);
    dsc1.putInteger(idLNR, 0);
    dsc1.putInteger(idVigA, 0);
    dsc1.putInteger(idSTSH, 0);
    dsc1.putInteger(idSTSS, 0);
    dsc1.putInteger(idSTHH, 0);
    dsc1.putInteger(idSTHS, 0);
    dsc1.putInteger(idSTB, 0);
    dsc1.putInteger(idPC_S, 0);
    dsc1.putInteger(idPC_D, 0);
    dsc1.putInteger(idPC_L, 0);
    dsc1.putInteger(idPC_H, 0);
    dsc1.putInteger(idPC_one, 25);
    dsc1.putInteger(idPC_two, 50);
    dsc1.putInteger(idPC_three, 75);
    dsc1.putDouble(idShpR, 1.000000);
    dsc1.putInteger(idShpD, 25);
    dsc1.putInteger(idShpM, 0);
    dsc1.putInteger(idPCVA, 0);
    dsc1.putInteger(idGRNA, 0);
    dsc1.putInteger(idLPEn, 0);
    dsc1.putInteger(idMDis, 0);
    dsc1.putInteger(idPerV, 0);
    dsc1.putInteger(idPerH, 0);
    dsc1.putDouble(idPerR, 0.000000);
    dsc1.putInteger(idPerS, 100);
    dsc1.putInteger(idPerA, 0);
    dsc1.putInteger(idPerU, 0);
    dsc1.putDouble(idPerX, 0.000000);
    dsc1.putDouble(idPerY, 0.000000);
    dsc1.putDouble(idExonetwo, 0.000000);
    dsc1.putInteger(idCronetwo, 100);
    dsc1.putInteger(idHionetwo, 0);
    dsc1.putInteger(idShonetwo, 0);
    dsc1.putInteger(idWhonetwo, 0);
    dsc1.putInteger(idBkonetwo, -80);
    dsc1.putInteger(idClonetwo, 0);
    dsc1.putInteger(idDhze, 100);
    dsc1.putInteger(idTMMs, 0);
    lst1.putInteger(0);
    lst1.putInteger(255);
    dsc1.putList(idCrv, lst1);
    lst2.putInteger(0);
    lst2.putInteger(255);
    dsc1.putList(idCrvR, lst2);
    lst3.putInteger(0);
    lst3.putInteger(255);
    dsc1.putList(idCrvG, lst3);
    lst4.putInteger(0);
    lst4.putInteger(255);
    dsc1.putList(idCrvB, lst4);
    dsc1.putInteger(idPrVe, 167772160);
    executeAction(idAdobeCameraRawFilter, dsc1, DialogModes.NO);
}

//----------------------------------------------------------------
// Add noise filter.
// amount: 0..400
//----------------------------------------------------------------
function addNoiseFilter(amount) {
    var dsc1 = new ActionDescriptor();

    var idAdNs = charIDToTypeID("AdNs");
    var idDstr = charIDToTypeID("Dstr");
    var idGsn = charIDToTypeID("Gsn ");
    var idNose = charIDToTypeID("Nose");
    var idPrc = charIDToTypeID("#Prc");
    var idMnch = charIDToTypeID("Mnch");
    var idFlRs = charIDToTypeID("FlRs");

    dsc1.putEnumerated(idDstr, idDstr, idGsn);
    dsc1.putUnitDouble(idNose, idPrc, amount);
    dsc1.putBoolean(idMnch, true);
    dsc1.putInteger(idFlRs, 64854829);
    executeAction(idAdNs, dsc1, DialogModes.NO);
}
